using System;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.IO;

namespace IQPD.Tests.PDF
{
	public enum PDFOption: int
	{
		ExtractToFile = 1,
		ExtractToClipboard = 2,
		ExtractToString = 3
	}


	/// <summary>
	/// Wrapper for Ingo Schmoekel PDFText.dll by H.Indenbirken.
	/// Based on the normal PDFText.dlls renamed to PDFText32.dll and PDFText64.dll.
	/// </summary>
	public static class PDFText
	{
		/// <summary>
		/// Extracts a PDF.
		/// </summary>
		/// <param name="filename">That's the pdf-file and can't be empty ;-)</param>
		/// <param name="opt">     opt=1 means extract the text-content to a textfile 
    ///                            (drive/path/file in “target”).
		///					               opt=2 means extract the text-content to clipboard.
		///					               opt=3 means extract the text-content to string.
		/// </param>
		/// <param name="hw">hw (default = 0)
		///		If this value is set to "1" only a hash-value (md5) will be returned. 
		///		You can use it to check if text-content was changed between two checks.
		///</param>
		/// <param name="fast">fast (default = 1)
		///		Fast means that the text-content of a page will be extracted similar 
    ///   to the original pdf-layout. 
		///		If you set this parameter to zero the text-content will be extracted 
    ///   string by string sorted.
		/// </param>
		/// <param name="target">If you want to extract into a textfile (opt=1) 
    ///   you should fill "target" with a new filename (with drive and path!). 
    ///   If you don't do this error 9006 will raise – telling you that the 
		///		target data is missing.
		/// </param>
		/// <param name="lspaces">lspaces (default = 0)
		///		lspaces means 1 to delete leading spaces on each text-line or 0.
		/// </param>
		/// <param name="ptitel">ptitel (default = '')
		///		ptitel means that pagenumbering should be inserted. If you insert 
    ///   here for example "page", 
		///		the pagenumbering will look like "page x / y". If you insert "Seite" 
    ///   it would be "Seite x / y". 
		///		If ptitel is empty there won’t be a pagenumbering on the extracted 
    ///   textcontent. 
		/// </param>
		/// <param name="pos">pos (default = 0)
		///		1 means that all extracted textstrings (fast=0) will have four leading 
    ///   values: current pagenumber, max. pagenumber, current row and column 
    ///   (in pixels). So you can get exactly the string position on a page. 
		///		Keep in mind that the highest row number is at the bottom of the page. 
		///		This parameter works only if "fast" is set to zero.
		/// </param>
		/// <param name="page">page (default = 0)
		///		If you don't want to extract the whole text of a document... if you 
    ///   have special known pages you want to extract, you can insert in "page" 
    ///   single page numbers. Then only these page will be extracted. 
		/// </param>
		/// <param name="clock">clock (default = 0)
		///		0 means no sandclock while the function is working...
		/// </param>
		/// <param name="blank">blank (default = 0)
		///		There are documents with justification layout. This means that 
    ///   sometimes (to fill each line) there are more than one space between 
    ///   words. If you're using PDFtext.dll to search in a second step through 
    ///   extracted textcontent it could be easier if you know that between 
    ///   words are only one space. If "blank" is set to 1 more than one space
		///		between words will be deleted.
		/// </param>
		/// <param name="ende">stop (default = 0)
		///		Sometimes you can get very voluminous documents with much different 
    ///   contents - The extraction can last many	seconds... minutes. Sometimes 
    ///   it's enough if you've extracted the first pages...  Here you can set 
    ///   a value that means "stop extracting after xxxx seconds". 0 means no 
    ///   limit, 1 until 1800 means from 1 second until 30 minutes (that should 
    ///   be enough).
		/// </param>
		/// <param name="wlist">wlist (default = 0)
		///		wlist means wordlist. If this parameter is set to "1". The extraction 
    ///   delivers word by word. Each word on a separate row.
		///	</param>
		/// <returns>
		/// Kinds of returned (error) codes for GetPDFText:
		///		0 = general/main error
		///		9001 = File not found
		///		9002 = No pdf-file
		///		9003 = There’s a user password
		///		9004 = Invalid/damaged page structure
		///		9005 = Target drive/path isn't valid
		///		9006 = Target drive/path is missing
		///		9007 = As opt-values only 1 (fileoutput), 2 (clipboardoutput) and
    ///          3 (stringoutput) are allowed
		///		9008 = Source and target (for fileoutput) is the same
		///		9015 = The text is based on the rare codepage 1251 ... extraction 
    ///          won't work in a proper way
		///		9016 = The text is based on the codepage CJK ... extraction won't 
    ///          work in a proper way
		///		   1 = Using opt=2 (clipboard) ... this means it's okay.
		///			     Using opt=1 (textfile)  ... this means it's okay.
		///		   9 = Means that there's no text (perhaps only images)
		/// </returns>
		/// 
		[DllImport (@"PDFtext64.dll", CallingConvention = CallingConvention.StdCall, 
                EntryPoint = "GetPDFText")]
	             [return: MarshalAs (UnmanagedType.LPWStr)]
               
		public static extern String GetPDFText64 ([MarshalAs (UnmanagedType.LPWStr)] 
                                              string filename, PDFOption opt,
		                                        	UInt32 hw, UInt32 fast, 
                                              [MarshalAs (UnmanagedType.LPWStr)] 
                                              string target, UInt32 lspaces,
		                                        	[MarshalAs (UnmanagedType.LPWStr)] 
                                              string ptitel, UInt32 pos, 
                                              UInt32 page, UInt32 clock,
			                                        UInt32 blank, UInt32 ende, 
                                              UInt32 wlist);

		[DllImport (@"PDFtext32.dll", CallingConvention = CallingConvention.StdCall, 
                EntryPoint = "GetPDFText")]
	             [return: MarshalAs (UnmanagedType.LPWStr)]

		public static extern String GetPDFText32 ([MarshalAs (UnmanagedType.LPWStr)] 
                                              string filename, PDFOption opt,
			                                        UInt32 hw, UInt32 fast, 
                                              [MarshalAs (UnmanagedType.LPWStr)] 
                                              string target, UInt32 lspaces,
			                                        [MarshalAs (UnmanagedType.LPWStr)] 
                                              string ptitel, UInt32 pos, 
                                              UInt32 page, UInt32 clock,
			                                        UInt32 blank, UInt32 ende, 
                                              UInt32 wlist);
 
		private const string Locking = "Lock"; // Used for thread locking.

		/// <summary>
		/// Extracts a PDF to file.
		/// </summary>
		/// <param name="fileName">That's the pdf-file and can't be empty</param>
		/// <param name="target">Path of the target file.</param>
		/// <param name="page">page (default = 0)
		/// If you don't want to extract the whole text of a document... 
		/// if you have special known pages you want to extract, you can insert 
    /// in "page" single page numbers. 
		/// Then only these page will be extracted. 
		/// </param>
		public static string Extract (string fileName, uint page = 0)
		{
			lock (Locking)
			{
				Debug.WriteLine 
             (Environment.Is64BitProcess ? "PDFtext64.dll" : "PDFtext32.dll");
		      		string result = Environment.Is64BitProcess ?
				GetPDFText64 (fileName, PDFOption.ExtractToString, 0, 1, null, 0,
						    		  string.Empty, 0, page, 0, 0, 0, 0)	:
				GetPDFText32 (fileName, PDFOption.ExtractToString, 0, 1, null, 0,
								      string.Empty, 0, page, 0, 0, 0, 0);
				switch (result)
				{
					case "0":
						throw new PDFTextException (fileName, "general/main error");
					case "9":
						throw new PDFTextException (fileName, "found no text");
					case "9001":
						throw new PDFTextException (fileName, "File not found");
					case "9002":
						throw new PDFTextException (fileName, "No pdf-file");
					case "9003":
						throw new PDFTextException (fileName, "There’s a user password");
					case "9004":
						throw new PDFTextException (fileName, "Invalid/damaged page structure.");
					case "9005":
						throw new PDFTextException (fileName, "Target isn't valid");
					case "9006":
						throw new PDFTextException (fileName, "Target drive/path is missing");
					case "9007":
						throw new PDFTextException (fileName, "As opt-values only 1 (fileoutput) and 2 (clipboardoutput) are allowed.");
					case "9008":
						throw new PDFTextException (fileName, "Source and target (for fileoutput) is the same");
					case "9015":
						throw new PDFTextException (fileName, "The text is based on the rare codepage 1251 ... extraction won't work proper.");
					case "9016":
						throw new PDFTextException (fileName, "The text is based on the codepage CJK ... extraction won't work proper.");
					default:
						return result;
				}
			}
		}
	}

	public class PDFTextException: System.Exception
	{
		private readonly string filename;

		/// <summary>
		/// Name of failed file
		/// </summary>
		public string Filename
		{
			get { return filename; }
		}

		/// <summary>
		/// Creates a new exception for the given file
		/// </summary>
		/// <param name="filename">Name of failed file</param>
		/// <param name="msg">Message for the exception</param>
		internal PDFTextException (string filename, string msg)
	  : base (string.Format ("GetPDFText {0}\r\n{1}", Path.GetFileName (filename), msg))
		{
			this.filename = filename;
		}
	}

}
